@@ -17,8 +17,9 @@ from steam_sdk.parsers.ParserProteCCT import ParserProteCCT
 from steam_sdk.parsers.ParserProtePyBBQ import ParserPyBBQ
 from steam_sdk.parsers.ParserRoxie import ParserRoxie
 from steam_sdk.parsers.ParserMap2d import getParametersFromMap2d
+from steam_sdk.parsers.dict_to_yaml import dict_to_yaml
 from steam_sdk.plotters import PlotterRoxie
-from steam_sdk.plotters import PlotterModel
+from steam_sdk.plotters.PlotterModel import PlotterModel
 
 
 class BuilderModel:
@@ -102,7 +103,8 @@ class BuilderModel:
 
                 if flag_plot_all:
                     PlotterRoxie.plot_all(self.roxie_data)
-                    PlotterModel.plot_all(self.model_data, self.roxie_data)
+                    PM = PlotterModel(self.roxie_data)
+                    PM.plot_all(self.model_data)
 
         ### Case of a circuit model
         elif case_model == 'circuit':
@@ -156,6 +158,10 @@ class BuilderModel:
 
         # Find folder where the input file is located, which will be used as the "anchor" for all input files
         self.path_magnet = Path(self.file_model_data).parent
+        self.path_data = None
+        self.path_map2d = None
+        self.path_cadata = None
+        self.path_iron = None
 
         if case_model == 'magnet':
             # Set a few paths relative to the "anchor" path
@@ -169,13 +175,13 @@ class BuilderModel:
                 self.path_cadata = Path(self.path_magnet / self.model_data.Sources.conductor_fromROXIE).resolve()
             if self.model_data.Sources.iron_fromROXIE:
                 self.path_iron = Path(self.path_magnet / self.model_data.Sources.iron_fromROXIE).resolve()
-        elif case_model in ['circuit', 'conductor']:
-            self.path_data   = None
-            self.path_map2d  = None
-            self.path_cadata = None
-            self.path_iron   = None
+        elif case_model in 'circuit':
+            pass  # no paths to assign
+        elif case_model == 'conductor':
+            if self.conductor_data.Sources.magnetic_field_fromROXIE:
+                self.path_map2d = Path(self.path_magnet / self.conductor_data.Sources.magnetic_field_fromROXIE).resolve()
         else:
-            raise Exception ('case_model ({}) no supported'.format(case_model))
+            raise Exception('case_model ({}) no supported'.format(case_model))
 
         # Set settings path
         # Find user name
@@ -262,6 +268,7 @@ class BuilderModel:
         roxie_parser = ParserRoxie()
         self.roxie_data = roxie_parser.getData(dir_data=self.path_data, dir_cadata=self.path_cadata, dir_iron=self.path_iron)
 
+
     def buildLEDET(self, case_model: str = 'magnet'):
         """
             Building a LEDET model
@@ -354,6 +361,10 @@ class BuilderModel:
             nameFileLEDET = os.path.join(self.output_path, magnet_name + '.xlsx')  # full path of the LEDET input file to write
             parser_ledet.writeLedet2Excel(full_path_file_name=nameFileLEDET, verbose=self.verbose, SkipConsistencyCheck=True)
 
+            # Write output yaml file
+            nameFileLedetYaml = os.path.join(self.output_path, magnet_name + '.yaml')  # full path of the LEDET input file to write
+            parser_ledet.write2yaml(full_path_file_name=nameFileLedetYaml, verbose=self.verbose, SkipConsistencyCheck=True)
+
             # Write output json file
             if self.flag_json:
                 nameFileLedetJson = os.path.join(self.output_path, magnet_name + '.json')  # full path of the LEDET input file to write
@@ -398,6 +409,10 @@ class BuilderModel:
             nameFileLEDET = os.path.join(self.output_path, conductor_name + '.xlsx')  # full path of the LEDET input file to write
             parser_ledet.writeLedet2Excel(full_path_file_name=nameFileLEDET, verbose=self.verbose, SkipConsistencyCheck=True)
 
+            # Write output yaml file
+            nameFileLedetYaml = os.path.join(self.output_path, conductor_name + '.yaml')  # full path of the LEDET input file to write
+            parser_ledet.write2yaml(full_path_file_name=nameFileLedetYaml, verbose=self.verbose, SkipConsistencyCheck=True)
+
             # Write output json file
             if self.flag_json:
                 nameFileLedetJson = os.path.join(self.output_path, conductor_name + '.json')  # full path of the LEDET input file to write
@@ -479,9 +494,9 @@ class BuilderModel:
             Path(self.dump_all_path).mkdir(parents=True)
 
         # Write output .yaml file
-        dump_all_full_path = os.path.join(dump_all_path, self.model_data.GeneralParameters.magnet_name +
-                                          '_all_data.yaml')
-        with open(dump_all_full_path, 'w') as outfile:
-            yaml.dump(all_data_dict, outfile, default_flow_style=False, sort_keys=False)
+        dump_all_full_path = os.path.join(dump_all_path, self.model_data.GeneralParameters.magnet_name + '_all_data.yaml')
+        # with open(dump_all_full_path, 'w') as outfile:
+        #     yaml.dump(all_data_dict, outfile, default_flow_style=False, sort_keys=False)
+        dict_to_yaml(all_data_dict, dump_all_full_path, list_exceptions=['Conductors'])
 
 
