@@ -1,5 +1,5 @@
 from pydantic import BaseModel
-from typing import (List, Union, Literal)
+from typing import (List, Union, Literal, Dict)
 
 
 ############################
@@ -58,7 +58,6 @@ class MakeModel(BaseModel):
     """
         Level 2: Analysis step to generate a model using BuilderModel
     """
-    step_name:         str = None  # this key will be called by AnalysisStepSequence
     type:              Literal['MakeModel']
     model_name:        str = None
     file_model_data:   str = None  # it would be nice if it could be read as parameters from other keys (anchors)
@@ -76,7 +75,6 @@ class ModifyModel(BaseModel):
     """
         Level 2: Analysis step to modify an existing BuilderModel object by changing one variable
     """
-    step_name:              str = None  # this key will be called by AnalysisStepSequence
     type:                   Literal['ModifyModel']
     model_name:             str = None
     variable_to_change:     str = None
@@ -90,7 +88,6 @@ class ModifyModelMultipleVariables(BaseModel):
     """
         Level 2: Analysis step to modify an existing BuilderModel object by changing a list of variables
     """
-    step_name:              str = None  # this key will be called by AnalysisStepSequence
     type:                   Literal['ModifyModelMultipleVariables']
     model_name:             str = None
     variables_to_change:    List[str] = []
@@ -101,7 +98,6 @@ class ModifyModelMultipleVariables(BaseModel):
     software:               List[str] = []
 
 class SetUpFolder(BaseModel):
-    step_name:  str = None  # this key will be called by AnalysisStepSequence
     type:       Literal['SetUpFolder']
     simulation_name: str = None
     software: List[str] = []
@@ -110,7 +106,6 @@ class RunSimulation(BaseModel):
     """
         Level 2: Analysis step to run a simulation file
     """
-    step_name:          str = None  # this key will be called by AnalysisStepSequence
     type:               Literal['RunSimulation']
     software:           str = None
     simulation_name:    str = None
@@ -121,15 +116,14 @@ class PostProcess(BaseModel):
     """
         Level 2: Analysis step to run a simulation file
     """
-    step_name:          str = None  # this key will be called by AnalysisStepSequence
     type:               Literal['PostProcess']
 
-class AnalysisStep(BaseModel):
-    """
-        Level 1: Class for information on the analysis step
-        Objects of this class will be defined in AnalysisStepDefinition
-    """
-    step: Union[MakeModel, ModifyModel, ModifyModelMultipleVariables, SetUpFolder, RunSimulation, PostProcess] = {}
+# class AnalysisStep(BaseModel):
+#     """
+#         Level 1: Class for information on the analysis step
+#         Objects of this class will be defined in AnalysisStepDefinition
+#     """
+#     step: Union[MakeModel, ModifyModel, ModifyModelMultipleVariables, SetUpFolder, RunSimulation, PostProcess] = {}
 
 ############################
 # Highest level
@@ -150,12 +144,5 @@ class DataAnalysis(BaseModel):
     GeneralParameters: General = General()
     PermanentSettings: PermanentSettings = PermanentSettings()
     WorkingFolders:    WorkingFolders = WorkingFolders()
-    AnalysisStepDefinition: List[AnalysisStep] = [
-        AnalysisStep(step={'type': 'MakeModel'}),
-        AnalysisStep(step={'type': 'ModifyModel'}),
-        AnalysisStep(step={'type': 'ModifyModelMultipleVariables'}),
-        AnalysisStep(step={'type': 'SetUpFolder'}),
-        AnalysisStep(step={'type': 'RunSimulation'}),
-        AnalysisStep(step={'type': 'PostProcess'}),
-    ]
+    AnalysisStepDefinition: Dict[str, Union[MakeModel, ModifyModel, ModifyModelMultipleVariables, SetUpFolder, RunSimulation, PostProcess]] = {}
     AnalysisStepSequence: List[str] = []  # Here the analysis steps are defined, in execution order. Names must be defined in AnalysisStepDefinition. Repetitions ARE allowed.
