@@ -6,6 +6,15 @@ from steam_sdk.data.DataConductor import Conductor
 from steam_sdk.data.DataModelMagnet import Circuit, PowerSupply, QuenchProtection, LEDET
 
 
+############################
+# Source files
+class SourceFiles(BaseModel):
+    """
+        Level 1: Class for the source files
+    """
+    magnetic_field_fromROXIE: str = None  # ROXIE .map2d file
+
+
 ############################
 # General parameters
 class Model(BaseModel):
@@ -101,6 +110,7 @@ class GeometryPyBBQ(BaseModel):
     """
     thInsul: float = None
     lenBusbar: float = None
+    tapetype: str = None
 
 
 class MagneticFieldPyBBQ(BaseModel):
@@ -124,6 +134,10 @@ class SimulationPyBBQ(BaseModel):
     output: bool = None
     dt: float = None
     t0: List[float] = []
+    posref: List[float] = []
+    print_every: int = None
+    store_every: int = None
+    plot_every: int = None
 
 class PhysicsPyBBQ(BaseModel):
     """
@@ -148,7 +162,9 @@ class PhysicsPyBBQ(BaseModel):
     tValidation: float = None
     TVQRef: float = None
     VThreshold: float = None
-    withCoolingToBath: float = None
+    withCoolingToBath: bool = None
+    withCoolingInternal: bool = None
+    Jc_4K_5T_NbTi: float = None
 
 
 class QuenchInitializationPyBBQ(BaseModel):
@@ -185,6 +201,7 @@ class DataModelConductor(BaseModel):
         :return: DataModelMagnet object
     '''
 
+    Sources: SourceFiles = SourceFiles()
     GeneralParameters: General = General()
     Conductors: List[Conductor] = [Conductor(cable={'type': 'Rutherford'}, strand={'type': 'Round'}, Jc_fit={'type': 'CUDI1'})]
     Circuit: Circuit = Circuit()
