@@ -13,10 +13,11 @@ class DataPyBBQ(BaseModel):
     CuSC: float = None
     non_void: float = None
     shape: str = None
+    strands: int = None # New
+    strand_dmt: float = None # New
     layers: int = None
     insulation_thickness: float = None
     busbar_length: float = None
-    sections: int = None
     T1: float = None
 
     # Magnetic Field
@@ -27,22 +28,42 @@ class DataPyBBQ(BaseModel):
     Self_Field: float = None
     B0_dump: bool = None
 
+    # Materials:
+    material: str = None            # New
+    tapetype: str = None            # New
+    RRR: float = None               # New
+    Jc_4K_5T_NbTi: float = None     # New
+
     # Load
     Current: float = None
     Inductance: float = None
     DumpR: float = None
 
     # Cooling
-    Helium_Cooling: bool = None
+    c5: float = None                # New
+    c6: float = None                # New
+    p: float = None                 # New
+    Pmax: float = None              # New
+    Helium_cooling: bool = None
+    Helium_cooling_internal: bool = None    # New
+    Wetted_p: float = None          # New
 
     # Protection and Detection
     Detection_Voltage: float = None
     Protection_Delay: float = None
 
+    # Analysis
+    Tref: float = None              # New
+    Posref: List[float] = None      # New
+
     # Solver setting:
     output: bool = None
     dt: float = None
     t0: List[float] = []
+    sections: int = None # Moved
+    print_every: int = None     # New
+    store_every: int = None     # New
+    plot_every: int = None      # New
+    sim_name: str = None        # New
+
 
-    # Materials:
-    matpath: str = None
