@@ -12,9 +12,12 @@ class DataSettingsSTEAM:
     JAVA_jdk_path:       str = None  # full path to folder with java jdk
     CFunLibPath:         str = None  # path to dll files with material properties
     LEDET_path:          str = None  #
+    PyBBQ_path:          str = None  #
     ProteCCT_path:       str = None  #
     PSPICE_path:         str = None  #
     COSIM_path:          str = None  #
     FiQuS_path:          str = None  #
     PSPICE_library_path: str = None  #
-    local_LEDET_folder:  str = None  # full path to local LEDET folder
\ No newline at end of file
+    local_LEDET_folder:  str = None  # full path to local LEDET folder
+    local_PyBBQ_folder:  str = None  # full path to local PyBBQ folder
+    local_PSPICE_folder: str = None  # full path to local PSPICE folder
\ No newline at end of file
