@@ -5,7 +5,8 @@ def lookupModelDataToLEDET(key: str, mode='data2ledet'):
     """
     lookup = {
         'GeneralParameters.T_initial': 'T00',
-        'GeneralParameters.magnetic_length': 'l_magnet',
+        'GeneralParameters.magnetic_length': 'l_magnet',  # useful when case_model='magnet'
+        'GeneralParameters.length_busbar': 'l_magnet',  # useful when case_model='conductor'
         'GeneralParameters.magnet_inductance.fL_I': 'fL_I',
         'GeneralParameters.magnet_inductance.fL_L': 'fL_L',
 
