@@ -4,6 +4,11 @@ import ntpath
 import shutil
 import textwrap
 from pathlib import Path
+import matplotlib.pyplot as plt
+from scipy.interpolate import UnivariateSpline
+from scipy.interpolate import interp1d
+import pandas as pd
+import numpy as np
 
 import yaml
 
@@ -742,7 +747,6 @@ def get_default_autoconverge_options():
     }
     return options_autoconverge
 
-
 def ComparePSPICEParameters(fileA, fileB):
     '''
         Compare all the variables imported from two PSPICE netlists
@@ -757,4 +761,241 @@ def ComparePSPICEParameters(fileA, fileB):
     flag_equal = pp_a.circuit_data == pp_b.circuit_data
     return flag_equal
 
-#######################  Helper functions - END  #######################
\ No newline at end of file
+#######################  Helper functions - END  #######################
+
+#######################  Interpolating Resistance ######################
+def InterpolateResistance(current_level: float, path_resources: str, n_apertures: int = 2, Type: str = 'Linear', plot_interpolation: bool = False):
+    '''
+    Function to interpolate a resistance based on given, pre-calculated resistance values at different current level.
+    2 different options are available: Linear or Spline interpolation. Interpolation is done at each timestep, with all
+    available resistance values
+
+    :param current_level: float, current level to be interpolated
+    :param path_resources: str, path to the file with pre-calculated values
+    :param n_apertures: int, Number of apertures to calculate a resistance for
+    :param Type: str, either Linear or Spline, type of interpolation
+    :param plot_interpolation: bool
+    :return [time, new_R1, new_R2]: list of np.array, [time vector, resistance values for Ap. 1, Ap. 2]
+    '''
+
+    if Type not in ['Linear', 'Spline']:
+        raise Exception(f'Type not understood. Chosen {Type} but only "Linear", "Spline" available.')
+
+    ## Read the resistance values from a given file
+    Intp_file = path_resources
+    Intp_data = pd.read_csv(Intp_file)
+    col_t = []
+    col_r1 = []
+    col_r2 = []
+
+    for col in Intp_data.columns:
+        try:
+            _ = int(col)
+            idx = Intp_data.columns.get_loc(col)
+            col_t.append(Intp_data.columns[idx]) # Time
+            col_r1.append(Intp_data.columns[idx + 1]) # Aperture 1
+            if n_apertures == 2: col_r2.append(Intp_data.columns[idx + 2]) # Aperture 2
+        except:
+            pass
+    data_R1 = Intp_data[col_r1].drop([0]).to_numpy().astype(float)
+    if n_apertures == 2:
+        data_R2 = Intp_data[col_r2].drop([0]).to_numpy().astype(float)
+    time = Intp_data[col_t[-1]].drop([0]).to_numpy().astype(float)
+
+    ## Interpolate new current level based on given values
+    new_R1 = []
+    new_R2 = []
+    current_level = np.array([current_level]).reshape(-1, 1)
+
+    x = np.array(col_t).astype(float).reshape(-1, 1)
+
+    # Loop through all time values, to obtain an interpolation for each time value
+    for k in range(time.shape[0]):
+        ## Start with interpolating values for R1 (aperture 1)
+        # Obtain all resistance values of R1, available in the current time step (drop NaN)
+        new_y = data_R1[k][~np.isnan(data_R1[k])].reshape(-1, )
+        # Obtain the respective current values to the resistances
+        new_x = x[~np.isnan(data_R1[k])].reshape(-1, )
+
+        # 1. Available option: Spline interpolation (deprecated as poor performance)
+        if Type == 'Spline':
+            new_x = new_x[::-1]
+            new_y = new_y[::-1]
+
+            if len(new_x) <= 3:
+                new_R1 = np.append(new_R1, np.nan)
+            else:
+                spl = UnivariateSpline(new_x, new_y)
+                new_R1 = np.append(new_R1, spl(current_level))
+        # 2. option: Linear interpolation
+        elif Type == 'Linear':
+            if current_level <= max(new_x):
+                f = interp1d(new_x, new_y)
+                new_R1 = np.append(new_R1, f(current_level))
+            else:
+                new_R1 = np.append(new_R1, np.nan)
+
+        if n_apertures == 2:
+            ## Repeat procedure for R2 (aperture 2)
+            new_y = data_R2[k][~np.isnan(data_R2[k])].reshape(-1, )
+            new_x = x[~np.isnan(data_R2[k])].reshape(-1, )
+            if Type == 'Spline':
+                new_x = new_x[::-1]
+                new_y = new_y[::-1]
+
+                if len(new_x) <= 3:
+                    new_R1 = np.append(new_R1, np.nan)
+                else:
+                    spl = UnivariateSpline(new_x, new_y)
+                    new_R2 = np.append(new_R2, spl(current_level))
+            elif Type == 'Linear':
+                try:
+                    f = interp1d(new_x, new_y)
+                    new_R2 = np.append(new_R2, f(current_level))
+                except:
+                    new_R2 = np.append(new_R2, np.nan)
+
+    # Plot interpolation if wanted
+    if plot_interpolation:
+        f = plt.figure(figsize=(17, 8))
+        plt.subplot(1, 2, 1)
+        plt.plot(time, new_R1)
+        leg = ["Interpolated-" + str(current_level)]
+        for i in range(data_R1.shape[1]):
+            plt.plot(time, data_R1[:, i])
+            leg.append(x[i][0])
+        plt.legend(leg)
+        plt.xlabel('Time [s]')
+        plt.ylabel('Resistance [Ohm]')
+        plt.title('R_CoilSection 1')
+        if n_apertures == 2:
+            plt.subplot(1, 2, 2)
+            plt.plot(time, new_R2)
+            leg = ["Interpolated-" + str(current_level)]
+            for i in range(data_R2.shape[1]):
+                plt.plot(time, data_R2[:, i])
+                leg.append(x[i][0])
+            plt.legend(leg)
+            plt.xlabel('Time [s]')
+            plt.ylabel('Resistance [Ohm]')
+            plt.title('R_CoilSection 2')
+        f.suptitle(str(current_level[0][0]) + 'A', fontsize=16)
+        plt.show()
+    if n_apertures == 2:
+        return [time, new_R1, new_R2]
+    else:
+        return [time, new_R1]
+
+
+def writeStimuliFromInterpolation(current_level: list, n_total_magnets: int, n_apertures: int, magnets: list, tShift: list, Outputfile: str, path_resources: str,
+                                  InterpolationType: str = 'Linear', type_stl: str = 'a', sparseTimeStepping: int = 100):
+    '''
+    Function to write a resistance stimuli for n apertures of a magnet for any current level. Resistance will be interpolated
+    from pre-calculated values (see InterpolateResistance for closer explanation). Stimuli is then written in a .stl file for PSPICE
+
+    :param current_level: list, all current level that shall be used for interpolation (each magnet has 1 current level)
+    :param n_total_magnets: int, Number of total magnets in the circuit (A stimuli will be written for each, non-quenching = 0)
+    :param n_apertures: int, Number of apertures per magnet. A stimuli will be written for each aperture for each magnet
+    :param magnets: list, magnet numbers for which the stimuli shall be written
+    :param tShift: list, time shift that needs to be applied to each stimuli
+    (e.g. if magnet 1 quenches at 0.05s, magnet 2 at 1s etc.), so that the stimuli are applied at the correct time in the simulation
+    :param Outputfile: str, name of the stimuli-file
+    :param path_resources: str, path to the file with pre-calculated values
+    :param Type: str, either Linear or Spline, type of interpolation
+    :param type_stl: str, how to write the stimuli file (either 'a' (append) or 'w' (write))
+    :param sparseTimeStepping: int, every x-th time value only a stimuli point is written (to reduce size of stimuli)
+    :return:
+    '''
+    if n_apertures>2: raise Exception('Maximum of 2 apertures supported yet.')
+    # Ensure consistency of inputs
+    if len(magnets) != len(tShift): raise Exception(f'Please provide a time shift for each magnet. '
+                                                    f'Size of magnet list is {len(magnets)}, but size of tShift is {len(tShift)}')
+
+    R1 = np.array([])
+    R2 = np.array([])
+    print("Interpolating Coil-Resistances")
+    # Interpolate data for each current level and ensure correct data format
+    for k in current_level:
+        if n_apertures == 2:
+            [time, data_R1, data_R2] = InterpolateResistance(k * 2, path_resources, n_apertures=n_apertures, Type = InterpolationType)
+        else:
+            [time, data_R1] = InterpolateResistance(k * 2, path_resources, n_apertures=n_apertures, Type=InterpolationType)
+        if not R1.size > 0:
+            R1 = data_R1[np.newaxis, ...]
+        else:
+            R1 = np.vstack((R1, data_R1))
+        if n_apertures == 2:
+            if not R2.size > 0:
+                R2 = data_R2[np.newaxis, ...]
+            else:
+                R2 = np.vstack((R2, data_R2))
+    stlString = ''
+
+    # Loop through all current level
+    for k in range(1, n_total_magnets+1):
+        if k in magnets:
+            index = magnets.index(k)
+            timeShift = tShift[index]
+            if timeShift < 0: timeShift = 0
+
+            # Start with generating stimulus for first aperture
+            stlString = stlString + f'\n.STIMULUS R_coil_{str(1)}_M{str(magnets[index])} ' \
+                                    f'PWL \n+ TIME_SCALE_FACTOR = 1 \n+ VALUE_SCALE_FACTOR = 1 \n'
+            stlString = stlString + "+ ( 0s, 0.0 )\n"
+            count = 0
+            # Set a resistance value in the stimuli for each sparseTimeStepping * TimeValue
+            for l in range(1, R1.shape[1] - 1):
+                if np.isnan(R1[index, l]): continue
+                # Ensure starting at 0 (No negative time in PSPICE allowed
+                if float(time[l]) < 0:  timeShift = timeShift + abs(float(time[l])) - 0.03
+                if float(time[l]) + timeShift < 0:
+                    tt = 0
+                else:
+                    tt = float(time[l]) + timeShift
+                # If every sparseTimeStepping* time value reached, write an entry
+                if count >= sparseTimeStepping:
+                    stlString = stlString + "+ ( " + str(tt) + "s, " + str(R1[index, l]) + " )\n"
+                    count = 0
+                count = count + 1
+            # Write last time value to be 10000 s, with last resistance (resistance is assumed to stay constant)
+            R1_last = R1[index]
+            R1_last = R1_last[~np.isnan(R1_last)]
+            stlString = stlString + "+ ( " + str(10000) + "s," + str(R1_last[-1]) + " ) \n"
+            stlString = stlString + " \n"
+
+            if n_apertures == 2:
+                # Repeat procedure for second aperture
+                stlString = stlString + f'\n.STIMULUS R_coil_{str(2)}_M{str(magnets[index])} ' \
+                                        f'PWL \n+ TIME_SCALE_FACTOR = 1 \n+ VALUE_SCALE_FACTOR = 1 \n'
+                stlString = stlString + "+ ( 0s, 0.0 )\n"
+                count = 0
+                for l in range(1, R2.shape[1] - 1):
+                    if np.isnan(R2[index, l]): continue
+                    if float(time[l]) < 0:  timeShift = timeShift + abs(float(time[l])) - 0.03
+                    if float(time[l]) + timeShift < 0:
+                        tt = 0
+                    else:
+                        tt = float(time[l]) + timeShift
+                    if count >= sparseTimeStepping:
+                        stlString = stlString + "+ ( " + str(tt) + "s, " + str(R2[index, l]) + " )\n"
+                        count = 0
+                    count = count + 1
+                R2_last = R2[index]
+                R2_last = R2_last[~np.isnan(R2_last)]
+                stlString = stlString + "+ ( " + str(10000) + "s," + str(R2_last[-1]) + " ) \n"
+                stlString = stlString + " \n"
+        else:
+            stlString = stlString + f'\n.STIMULUS R_coil_{str(1)}_M{str(k)} ' \
+                                    f'PWL \n+ TIME_SCALE_FACTOR = 1 \n+ VALUE_SCALE_FACTOR = 1 \n'
+            stlString = stlString + "+ ( 0s, 0.0 )\n"
+            stlString = stlString + "+ ( 10000s, 0.0 )\n"
+
+            if n_apertures == 2:
+                stlString = stlString + f'\n.STIMULUS R_coil_{str(2)}_M{str(k)} ' \
+                                        f'PWL \n+ TIME_SCALE_FACTOR = 1 \n+ VALUE_SCALE_FACTOR = 1 \n'
+                stlString = stlString + "+ ( 0s, 0.0 )\n"
+                stlString = stlString + "+ ( 10000s, 0.0 )\n"
+
+    # Write the stimuli as a txt(stl) file
+    with open(Outputfile, type_stl) as ofile:
+        ofile.write(stlString)
