@@ -7,7 +7,7 @@ import yaml
 from pydantic import BaseModel, PrivateAttr
 from typing import (Deque, Dict, FrozenSet, List, Optional, Sequence, Set, Tuple, Union, Type)
 
-from steam_sdk.builders import geometricFunctions
+from steam_sdk.builders import geometricFunctions as gf
 from steam_sdk.data import DataRoxieParser as pd
 # import matplotlib.pyplot as plt
 # import matplotlib.lines as lines
@@ -71,76 +71,38 @@ def evalContent(s, g):
     return value
 
 
-def get_conductor_corners(roxie_data: pd.APIdata):
-    """ Returns conductor corner positions (insulated and bare)  and conductor currents in list form """
-    x_insulated = []
-    y_insulated = []
-    x_bare = []
-    y_bare = []
-    i_conductor = []
-    x_strand = []
-    y_strand = []
-    i_strand = []
-    strand_to_halfTurn = []
-    idx_half_turn = 0
-    for eo in roxie_data.coil.electrical_order:
-        winding = roxie_data.coil.coils[eo.coil].poles[eo.pole].layers[eo.layer].windings[eo.winding]
-        block = winding.blocks[eo.block]
-        for halfTurn_nr, halfTurn in block.half_turns.items():
-            idx_half_turn = idx_half_turn + 1
-            insulated = halfTurn.corners.insulated
-            bare = halfTurn.corners.bare
-            x_insulated.append([insulated.iL.x, insulated.oL.x, insulated.oR.x, insulated.iR.x])
-            y_insulated.append([insulated.iL.y, insulated.oL.y, insulated.oR.y, insulated.iR.y])
-            x_bare.append([bare.iL.x, bare.oL.x, bare.oR.x, bare.iR.x])
-            y_bare.append([bare.iL.y, bare.oL.y, bare.oR.y, bare.iR.y])
-            i_conductor.append(winding.cable_current * block.current_sign)
+class RoxieList:
+    def __init__(self, roxie_data):
+        """ Returns conductor corner positions (insulated and bare) and conductor currents in list form """
+        self.x_insulated = []
+        self.y_insulated = []
+        self.x_bare = []
+        self.y_bare = []
+        self.i_conductor = []
+        self.x_strand = []
+        self.y_strand = []
+        self.i_strand = []
+        self.strand_to_halfTurn = []
+        self.idx_half_turn = 0
+        for eo in roxie_data.coil.electrical_order:
+            winding = roxie_data.coil.coils[eo.coil].poles[eo.pole].layers[eo.layer].windings[eo.winding]
+            block = winding.blocks[eo.block]
+            for halfTurn_nr, halfTurn in block.half_turns.items():
+                self.idx_half_turn = self.idx_half_turn + 1
+                insulated = halfTurn.corners.insulated
+                bare = halfTurn.corners.bare
+                self.x_insulated.append([insulated.iL.x, insulated.oL.x, insulated.oR.x, insulated.iR.x])
+                self.y_insulated.append([insulated.iL.y, insulated.oL.y, insulated.oR.y, insulated.iR.y])
+                self.x_bare.append([bare.iL.x, bare.oL.x, bare.oR.x, bare.iR.x])
+                self.y_bare.append([bare.iL.y, bare.oL.y, bare.oR.y, bare.iR.y])
+                self.i_conductor.append(winding.cable_current * block.current_sign)
 
-            for strand_gr_nr, strand_gr in halfTurn.strand_groups.items():
-                for strand_nr, strand in strand_gr.strand_positions.items():
-                    x_strand.append(strand.x)
-                    y_strand.append(strand.y)
-                    i_strand.append(winding.strand_current * block.current_sign)
-                    strand_to_halfTurn.append(idx_half_turn)
-
-    return x_insulated, y_insulated, x_bare, y_bare, i_conductor, x_strand, y_strand, i_strand, strand_to_halfTurn
-
-
-def get_conductor_corners_old(roxie_data: pd.APIdata):
-    """ Returns conductor corner positions (insulated and bare)  and conductor currents in list form """
-    x_insulated = []
-    y_insulated = []
-    x_bare = []
-    y_bare = []
-    i_conductor = []
-    x_strand = []
-    y_strand = []
-    i_strand = []
-    strandToHalfTurn = []
-    idx_half_turn = 0
-    for coil_nr, coil in roxie_data.coil.coils.items():
-        for pole_nr, pole in coil.poles.items():
-            for layer_nr, layer in pole.layers.items():
-                for winding_key, winding in layer.windings.items():
-                    for block_key, block in winding.blocks.items():
-                        for halfTurn_nr, halfTurn in block.half_turns.items():
-                            idx_half_turn = idx_half_turn + 1
-                            insulated = halfTurn.corners.insulated
-                            bare = halfTurn.corners.bare
-                            x_insulated.append([insulated.iL.x, insulated.oL.x, insulated.oR.x, insulated.iR.x])
-                            y_insulated.append([insulated.iL.y, insulated.oL.y, insulated.oR.y, insulated.iR.y])
-                            x_bare.append([bare.iL.x, bare.oL.x, bare.oR.x, bare.iR.x])
-                            y_bare.append([bare.iL.y, bare.oL.y, bare.oR.y, bare.iR.y])
-                            i_conductor.append(winding.cable_current * block.current_sign)
-
-                            for strand_gr_nr, strand_gr in halfTurn.strand_groups.items():
-                                for strand_nr, strand in strand_gr.strand_positions.items():
-                                    x_strand.append(strand.x)
-                                    y_strand.append(strand.y)
-                                    i_strand.append(winding.strand_current * block.current_sign)
-                                    strandToHalfTurn.append(idx_half_turn)
-
-    return x_insulated, y_insulated, x_bare, y_bare, i_conductor, x_strand, y_strand, i_strand, strandToHalfTurn
+                for strand_gr_nr, strand_gr in halfTurn.strand_groups.items():
+                    for strand_nr, strand in strand_gr.strand_positions.items():
+                        self.x_strand.append(strand.x)
+                        self.y_strand.append(strand.y)
+                        self.i_strand.append(winding.strand_current * block.current_sign)
+                        self.strand_to_halfTurn.append(self.idx_half_turn)
 
 
 class ParserRoxie:
@@ -283,7 +245,7 @@ class ParserRoxie:
                                                 (abs(alpha) > np.pi / 2) & (alpha > 0),
                                                 (abs(alpha) < np.pi / 2) & (alpha < 0),
                                                 (abs(alpha) > np.pi / 2) & (alpha < 0)]
-                                        line_a = geometricFunctions.findLineThroughTwoPoints(
+                                        line_a = gf.findLineThroughTwoPoints(
                                             A, [((case[1] | case[2]) - (case[0] | case[3])) * A[1] /
                                                 np.tan((case[1] | case[3]) * np.pi +
                                                        ((case[0] | case[3]) - (case[1] | case[2])) * alpha) + A[0], 0.])
@@ -295,7 +257,7 @@ class ParserRoxie:
                                                 (abs(beta) > np.pi / 2) & (beta > 0),
                                                 (abs(beta) < np.pi / 2) & (beta < 0),
                                                 (abs(beta) > np.pi / 2) & (beta < 0)]
-                                        line_b = geometricFunctions.findLineThroughTwoPoints(
+                                        line_b = gf.findLineThroughTwoPoints(
                                             B, [((case[1] | case[2]) - (case[0] | case[3])) * B[1] /
                                                 np.tan((case[1] | case[3]) * np.pi +
                                                        ((case[0] | case[3]) - (case[1] | case[2])) * beta) + B[0], 0.])
@@ -448,7 +410,7 @@ class ParserRoxie:
                         elif j[:5] == 'Lmesh':
                             arguments = state0[6:-1].split(',')
                             name = arguments[0].strip()
-                            if name in corners:
+                            if name in corners or name in notches:
                                 data.hyper_lines[name + 'a'].elements = round(int(arguments[1].strip()) / 2)
                                 data.hyper_lines[name + 'b'].elements = round(int(arguments[1].strip()) / 2)
                             elif name in bars:
@@ -1256,11 +1218,10 @@ class ParserRoxie:
 
                 # Find line through points 1 and 4 of the current conductor (top-left and top-right)
                 # A, B, and C coefficients of the line, as in: A*x + B*y + C = 0
-                line = geometricFunctions.findLineThroughTwoPoints([xCable[0], yCable[0]], [xCable[3], yCable[3]],
-                                                                   verbose=verbose)
+                line = gf.findLineThroughTwoPoints([xCable[0], yCable[0]], [xCable[3], yCable[3]], verbose=verbose)
 
                 # Find the intersection points between the circle and the line just defined
-                xy = geometricFunctions.intersectLineCircle(line, circle, verbose=verbose)
+                xy = gf.intersectLineCircle(line, circle, verbose=verbose)
 
                 # Find the one of two intersection points that is closest to the x2 point of the current conductor
                 if xy[0] == [None, None] and xy[1] == [None, None]:
@@ -1310,10 +1271,9 @@ class ParserRoxie:
             if self.block.type == 2:  # block-coil
                 # Apply conductor rotation of angle=alpha around origin (x0Cond,y0Cond)
                 for i, arg in enumerate(xBareCable):
-                    (xBareCable[i], yBareCable[i]) = geometricFunctions.rotatePoint((xBareCable[i], yBareCable[i]),
-                                                                                    (radius, y0Cond), alpha)
-                    (xCable[i], yCable[i]) = geometricFunctions.rotatePoint((xCable[i], yCable[i]),
-                                                                            (radius, y0Cond), alpha)
+                    (xBareCable[i], yBareCable[i]) = gf.rotatePoint((xBareCable[i], yBareCable[i]),
+                                                                    (radius, y0Cond), alpha)
+                    (xCable[i], yCable[i]) = gf.rotatePoint((xCable[i], yCable[i]), (radius, y0Cond), alpha)
 
             # Mirror conductor about the X-axis
             if self.block.imag == 1:
@@ -1327,15 +1287,13 @@ class ParserRoxie:
 
             for i, arg in enumerate(xBareCable):
                 # Apply conductor rotation of angle=turn around origin (x0,y0)
-                (xBareCable[i], yBareCable[i]) = geometricFunctions.rotatePoint((xBareCable[i], yBareCable[i]),
-                                                                                (x0, y0), turn)
-                (xCable[i], yCable[i]) = geometricFunctions.rotatePoint((xCable[i], yCable[i]), (x0, y0), turn)
+                (xBareCable[i], yBareCable[i]) = gf.rotatePoint((xBareCable[i], yBareCable[i]), (x0, y0), turn)
+                (xCable[i], yCable[i]) = gf.rotatePoint((xCable[i], yCable[i]), (x0, y0), turn)
 
                 # Apply roll2 counterclockwise rotation transformation
-                (xBareCable[i], yBareCable[i]) = geometricFunctions.rotatePoint((xBareCable[i], yBareCable[i]),
-                                                                                (x0Roll, y0Roll), alphaRoll)
-                (xCable[i], yCable[i]) = geometricFunctions.rotatePoint((xCable[i], yCable[i]), (x0Roll, y0Roll),
-                                                                        alphaRoll)
+                (xBareCable[i], yBareCable[i]) = gf.rotatePoint((xBareCable[i], yBareCable[i]),
+                                                                (x0Roll, y0Roll), alphaRoll)
+                (xCable[i], yCable[i]) = gf.rotatePoint((xCable[i], yCable[i]), (x0Roll, y0Roll), alphaRoll)
 
                 # Apply shift2 cartesian shift transformation
                 xBareCable[i], yBareCable[i] = xBareCable[i] + shiftX, yBareCable[i] + shiftY
@@ -1366,12 +1324,22 @@ class ParserRoxie:
                 corner3 = [xCable[3], yCable[3]]  # 4
                 corner1 = [xCable[1], yCable[1]]  # 2
                 corner2 = [xCable[2], yCable[2]]  # 3
+            elif c == self.block.nco - 1 and c == 1:
+                corner0 = [xCable[0], yCable[0]]  # 1
+                corner3 = [xCable[3], yCable[3]]  # 4
+                corner_aux_i_next = [xCable[1], yCable[1]]
+                corner_aux_o_next = [xCable[2], yCable[2]]
             elif c == self.block.nco - 1:
                 corner0 = [xCable[0], yCable[0]]  # 1
                 corner3 = [xCable[3], yCable[3]]  # 4
+            elif c == 1:
+                corner_aux_i_next = [xCable[1], yCable[1]]
+                corner_aux_o_next = [xCable[2], yCable[2]]
             elif c == 0:
                 corner1 = [xCable[1], yCable[1]]  # 2
                 corner2 = [xCable[2], yCable[2]]  # 3
+                corner_aux_i = [xCable[0], yCable[0]]
+                corner_aux_o = [xCable[3], yCable[3]]
             else:
                 pass
 
@@ -1423,34 +1391,96 @@ class ParserRoxie:
             block.block_corners.iR = pd.Coord(x=sigDig(corner1[0]), y=sigDig(corner1[1]))  # 2
             block.block_corners.oR = pd.Coord(x=sigDig(corner2[0]), y=sigDig(corner2[1]))  # 3
         else:
-            new_corners_inner = geometricFunctions.intersectLineCircle(
-                geometricFunctions.findLineThroughTwoPoints(corner0, corner3),
-                [radius, coil.bore_center.x, coil.bore_center.y])
+            new_corners_inner = gf.intersectLineCircle(gf.findLineThroughTwoPoints(corner0, corner3),
+                                                       [radius, coil.bore_center.x, coil.bore_center.y])
             if min(abs(new_corners_inner[0][0] - corner0[0]),
                    abs(new_corners_inner[1][0] - corner0[0])) == abs(new_corners_inner[0][0] - corner0[0]):
                 new_inner = new_corners_inner[0]
             else:
                 new_inner = new_corners_inner[1]
-            outer_radius = (np.sqrt(np.square(corner2[0] - coil.bore_center.x) +
-                                    np.square(corner2[1] - coil.bore_center.y)))
-            new_corners_outer = geometricFunctions.intersectLineCircle(
-                geometricFunctions.findLineThroughTwoPoints(corner0, corner3),
+            mid_inner_end = [(corner0[0] + new_inner[0]) / 2, (corner0[1] + new_inner[1]) / 2]
+            inner_line_mid_point = [(corner1[0] + corner_aux_i[0]) / 2, (corner1[1] + corner_aux_i[1]) / 2]
+            mid_point_next = [(corner_aux_i_next[0] + corner_aux_i[0]) / 2,
+                              (corner_aux_i_next[1] + corner_aux_i[1]) / 2]
+            mid_inner_beg = gf.intersectTwoLines(gf.findLineThroughTwoPoints(mid_point_next, inner_line_mid_point),
+                                                 gf.findLineThroughTwoPoints(corner1, corner2))
+
+            # new_inner_radius = np.sqrt(np.square(mid_inner_end[0] - coil.bore_center.x) +
+            #                            np.square(mid_inner_end[1] - coil.bore_center.y))
+            # new_corners_inner_mid = gf.intersectLineCircle(
+            #     gf.findLineThroughTwoPoints(corner1, corner2),
+            #     [new_inner_radius, coil.bore_center.x, coil.bore_center.y])
+            # if min(abs(new_corners_inner_mid[0][0] - corner1[0]),
+            #        abs(new_corners_inner_mid[1][0] - corner1[0])) == abs(new_corners_inner_mid[0][0] - corner1[0]):
+            #     mid_inner_beg = new_corners_inner_mid[0]
+            # else:
+            #     mid_inner_beg = new_corners_inner_mid[1]
+
+            outer_radius = np.sqrt(np.square(corner2[0] - coil.bore_center.x) +
+                                   np.square(corner2[1] - coil.bore_center.y))
+            new_corners_outer = gf.intersectLineCircle(
+                gf.findLineThroughTwoPoints(corner0, corner3),
                 [outer_radius, coil.bore_center.x, coil.bore_center.y])
-            if min(abs(new_corners_outer[0][0] - corner0[0]),
-                   abs(new_corners_outer[1][0] - corner0[0])) == abs(new_corners_outer[0][0] - corner0[0]):
+            if min(abs(new_corners_outer[0][0] - corner3[0]),
+                   abs(new_corners_outer[1][0] - corner3[0])) == abs(new_corners_outer[0][0] - corner3[0]):
                 new_outer = new_corners_outer[0]
             else:
                 new_outer = new_corners_outer[1]
+            mid_outer_end = [(corner3[0] + new_outer[0]) / 2, (corner3[1] + new_outer[1]) / 2]
+            outer_line_mid_point = [(corner2[0] + corner_aux_o[0]) / 2, (corner2[1] + corner_aux_o[1]) / 2]
+            mid_point_next = [(corner_aux_o_next[0] + corner_aux_o[0]) / 2,
+                              (corner_aux_o_next[1] + corner_aux_o[1]) / 2]
+            mid_outer_beg = gf.intersectTwoLines(gf.findLineThroughTwoPoints(mid_point_next, outer_line_mid_point),
+                                                 gf.findLineThroughTwoPoints(corner1, corner2))
+            # new_outer_radius = np.sqrt(np.square(mid_outer_end[0] - coil.bore_center.x) +
+            #                            np.square(mid_outer_end[1] - coil.bore_center.y))
+            # new_corners_outer_mid = gf.intersectLineCircle(
+            #     gf.findLineThroughTwoPoints(corner1, corner2),
+            #     [new_outer_radius, coil.bore_center.x, coil.bore_center.y])
+            # if min(abs(new_corners_outer_mid[0][0] - corner2[0]),
+            #        abs(new_corners_outer_mid[1][0] - corner2[0])) == abs(new_corners_outer_mid[0][0] - corner2[0]):
+            #     mid_outer_beg = new_corners_outer_mid[0]
+            # else:
+            #     mid_outer_beg = new_corners_outer_mid[1]
+
             if corner0[0] < corner1[0]:
-                block.block_corners.iL = pd.Coord(x=sigDig(new_inner[0]), y=sigDig(new_inner[1]))  # 1
-                block.block_corners.oL = pd.Coord(x=sigDig(new_outer[0]), y=sigDig(new_outer[1]))  # 4
-                block.block_corners.iR = pd.Coord(x=sigDig(corner1[0]), y=sigDig(corner1[1]))  # 2
-                block.block_corners.oR = pd.Coord(x=sigDig(corner2[0]), y=sigDig(corner2[1]))  # 3
+                block.block_corners.iL = pd.Coord(x=sigDig(mid_inner_end[0]), y=sigDig(mid_inner_end[1]))  # 1
+                block.block_corners.oL = pd.Coord(x=sigDig(mid_outer_end[0]), y=sigDig(mid_outer_end[1]))  # 4
+                block.block_corners.iR = pd.Coord(x=sigDig(mid_inner_beg[0]), y=sigDig(mid_inner_beg[1]))  # 2
+                block.block_corners.oR = pd.Coord(x=sigDig(mid_outer_beg[0]), y=sigDig(mid_outer_beg[1]))  # 3
             else:
-                block.block_corners.iR = pd.Coord(x=sigDig(new_inner[0]), y=sigDig(new_inner[1]))
-                block.block_corners.oR = pd.Coord(x=sigDig(new_outer[0]), y=sigDig(new_outer[1]))
-                block.block_corners.iL = pd.Coord(x=sigDig(corner1[0]), y=sigDig(corner1[1]))
-                block.block_corners.oL = pd.Coord(x=sigDig(corner2[0]), y=sigDig(corner2[1]))
+                block.block_corners.iR = pd.Coord(x=sigDig(mid_inner_end[0]), y=sigDig(mid_inner_end[1]))
+                block.block_corners.oR = pd.Coord(x=sigDig(mid_outer_end[0]), y=sigDig(mid_outer_end[1]))
+                block.block_corners.iL = pd.Coord(x=sigDig(mid_inner_beg[0]), y=sigDig(mid_inner_beg[1]))
+                block.block_corners.oL = pd.Coord(x=sigDig(mid_outer_beg[0]), y=sigDig(mid_outer_beg[1]))
+            # new_corners_inner = gf.intersectLineCircle(
+            #     gf.findLineThroughTwoPoints(corner0, corner3),
+            #     [radius, coil.bore_center.x, coil.bore_center.y])
+            # if min(abs(new_corners_inner[0][0] - corner0[0]),
+            #        abs(new_corners_inner[1][0] - corner0[0])) == abs(new_corners_inner[0][0] - corner0[0]):
+            #     new_inner = new_corners_inner[0]
+            # else:
+            #     new_inner = new_corners_inner[1]
+            # outer_radius = (np.sqrt(np.square(corner2[0] - coil.bore_center.x) +
+            #                         np.square(corner2[1] - coil.bore_center.y)))
+            # new_corners_outer = gf.intersectLineCircle(
+            #     gf.findLineThroughTwoPoints(corner0, corner3),
+            #     [outer_radius, coil.bore_center.x, coil.bore_center.y])
+            # if min(abs(new_corners_outer[0][0] - corner0[0]),
+            #        abs(new_corners_outer[1][0] - corner0[0])) == abs(new_corners_outer[0][0] - corner0[0]):
+            #     new_outer = new_corners_outer[0]
+            # else:
+            #     new_outer = new_corners_outer[1]
+            # if corner0[0] < corner1[0]:
+            #     block.block_corners.iL = pd.Coord(x=sigDig(new_inner[0]), y=sigDig(new_inner[1]))  # 1
+            #     block.block_corners.oL = pd.Coord(x=sigDig(new_outer[0]), y=sigDig(new_outer[1]))  # 4
+            #     block.block_corners.iR = pd.Coord(x=sigDig(corner1[0]), y=sigDig(corner1[1]))  # 2
+            #     block.block_corners.oR = pd.Coord(x=sigDig(corner2[0]), y=sigDig(corner2[1]))  # 3
+            # else:
+            #     block.block_corners.iR = pd.Coord(x=sigDig(new_inner[0]), y=sigDig(new_inner[1]))
+            #     block.block_corners.oR = pd.Coord(x=sigDig(new_outer[0]), y=sigDig(new_outer[1]))
+            #     block.block_corners.iL = pd.Coord(x=sigDig(corner1[0]), y=sigDig(corner1[1]))
+            #     block.block_corners.oL = pd.Coord(x=sigDig(corner2[0]), y=sigDig(corner2[1]))
 
         # ((block.pole % 2 == 0) * np.sign(-I[0]) +
         #  (block.pole % 2 != 0) * np.sign(I[0])) * block.radius / 1e3]
