@@ -1,4 +1,6 @@
 import os
+from pathlib import Path
+
 import numpy as np
 import pandas as pd
 
@@ -201,16 +203,21 @@ class ParserTdms:
         return dict_groups_signals
 
 
-    def convertTdmsToCsv(self, path_output: str, selected_groups: list = []):
+    def convertTdmsToCsv(self, path_output, selected_groups: list = []):
         '''
         Convert the signals of one or more groups of a .tdms file into a .csv file
         Each signal group will be converted to a different .csv file
 
-        :param path_output: Full path of the output file(s). Note: for each group, a different suffix will be added to the file name
+        :param path_output: Full path of the output file(s), as a string or Path. Note: for each group, a different suffix will be added to the file name
+        #TODO Add only two type options for path_output
         :param selected_groups: List of groups to convert. If left empty, all groups will be converted
         :return:
         '''
 
+        # If path_output is given as a string, resolve its path
+        if type(path_output) == str:
+            path_output = Path(path_output).resolve()
+
         # If the output folder does not exist, make it now
         make_folder_if_not_existing(os.path.dirname(path_output))
 
@@ -226,7 +233,7 @@ class ParserTdms:
         for group in selected_groups:
             # Define current file name
             suffix_group = f'_{group}'
-            full_path_file_group = path_output.split('.csv')[0] + suffix_group + '.csv'
+            full_path_file_group = str(path_output).split('.csv')[0] + suffix_group + '.csv'
 
             # Write file
             self.dict_groups_signals[group].to_csv(full_path_file_group, index=False, sep=',')  # , float_format='%.7f')  # Write file
